/**
 * command line parser
 *
 * @author Manuke
 */

package getopt;

// import

import java.util.*;

// class implementation

public class GetOpt {
	// type

	public static class Result {
		public ArrayList<String[]> opts = new ArrayList<String[]>();
		public ArrayList<String> args = new ArrayList<String>();
	}

	// attribute

	Map<String, Boolean> omap_ = new HashMap<String, Boolean>();

	// create & destroy

	public GetOpt(String sopt, String[] lopt) {
		if (sopt != null) {
			for (int i = 0; i < sopt.length(); i++) {
				String label = "-"+sopt.charAt(i);
				boolean hasParam = false;
				if ((i+1 < sopt.length()) &&
					(sopt.charAt(i+1) == ':'))
				{
					i++;
					hasParam = true;
				}
				omap_.put(label, hasParam);
			}
		}
		if (lopt != null) {
			for (int i = 0; i < lopt.length; i++) {
				String label = "--"+lopt[i];
				boolean hasParam = false;
				if (label.endsWith("=")) {
					label = label.substring(0, label.length()-1);
					hasParam = true;
				}
				omap_.put(label, hasParam);
			}
		}
	}

	public GetOpt(String sopt, String[] lopt, String[] args) {
		this(sopt, lopt);
		decode(args);
	}

	// operation

	public Result decode(String[] args) {
		Result goresult = new Result();
		for (int i = 0; i < args.length; i++) {
			String arg = args[i];
			if (arg.equals("--")) {
				while (++i < args.length) {
					goresult.args.add(args[i]);
				}
			} else if (
				(arg.length() >= 2) &&
				(arg.charAt(0) == '-') &&
				(arg.charAt(1) != '-'))
			{
				for (int j = 1; j < arg.length(); j++) {
					String optLabel = "-"+arg.charAt(j),
						optParam = null;
					if (!omap_.containsKey(optLabel)) {
						throw new RuntimeException(
							String.format(
								"option %s not recognized",
								optLabel));
					}
					boolean hasParam = omap_.get(optLabel);
					if (!hasParam) {
						// ok
					} else {
						if (j+1 < arg.length()) {
							optParam = arg.substring(j+1);
							j = arg.length()-1;
						}
						if ((optParam == null) &&
							(i+1 < args.length))
						{
							optParam = args[++i];
						}
						if (optParam == null) {
							throw new RuntimeException(
								String.format(
									"option %s requires parameter",
									optLabel));
						}
					}
					goresult.opts.add(new String[] {optLabel, optParam});
				}
			} else if (
				(arg.length() >= 3) &&
				arg.startsWith("--"))
			{
				String optLabel = arg,
					optParam = null;
				int sep = optLabel.indexOf('=');
				if (sep >= 0) {
					optParam = optLabel.substring(sep+1);
					optLabel = optLabel.substring(0, sep);
				}
				if (!omap_.containsKey(optLabel)) {
					ArrayList<String> matched = new ArrayList<String>();
					for (String optLabel2 : omap_.keySet()) {
						if (optLabel2.startsWith(optLabel)) {
							matched.add(optLabel2);
						}
					}
					if (matched.size() == 1) {
						optLabel = matched.get(0);
					} else {
						throw new RuntimeException(
							String.format(
								"option {0} not recognized",
								optLabel));
					}
				}
				boolean hasParam = omap_.get(optLabel);
				if (!hasParam) {
					if (optParam != null) {
						throw new RuntimeException(
							String.format(
								"option {0} must not have parameter",
								optLabel));
					}
				} else {
					if ((optParam == null) &&
						(i+1 < args.length))
					{
						optParam = args[++i];
					}
					if (optParam == null) {
						throw new RuntimeException(
							String.format(
								"option {0} requires parameter",
								optLabel));
					}
				}
				goresult.opts.add(new String[] {optLabel, optParam});
			} else {
				goresult.args.add(arg);
			}
		}
		return goresult;
	}
}
