/**
 * GSweeper generic button
 *
 * @author Manuke
 */

package gsweeper;

// import

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.event.*;

// class implementation

class GSButton
	extends GSComponent
{
	// type

	public class ButtonEvent
		extends EventObject
	{
		// attribute

		public String name;
		public boolean stat;

		// create & destroy

		public ButtonEvent(
			Object source,
			String name, boolean stat)
		{
			super(source);
			this.name = name;
			this.stat = stat;
		}
	}

	public interface ButtonListener
		extends EventListener
	{
		// button listener

		public void buttonClicked(ButtonEvent e);
	}

	// constant

	public static final int CX_BUTTONBORDER = 4;
	public static final int CY_BUTTONBORDER = 4;
	public static final int CX_ICONPUSHOFS = 1;
	public static final int CY_ICONPUSHOFS = 1;

	public static final Color BORDER_HIGHLIGHT_COLOR1 =
		new Color(0xFFFFFF);
	public static final Color BORDER_HIGHLIGHT_COLOR2 =
		new Color(0xFCFCFC);
	public static final Color BORDER_SHADOW_COLOR1 =
		new Color(0xA0A0A0);
	public static final Color BORDER_SHADOW_COLOR2 =
		new Color(0x606060);
	public static final Color FACE_COLOR1 =
		new Color(0xF8F8F8);
	public static final Color FACE_COLOR2 =
		new Color(0xC0C0C0);

	// attribute

	boolean toggle_;
	boolean pushed_ = false;
	BufferedImage buttonIcon_ = null;

	EventListenerList buttonListeners_ =
		new EventListenerList();

	// create & destroy

	public GSButton(
		boolean toggle,
		BufferedImage buttonIcon,
		Dimension borderDim)
	{
		setOpaque(true);
		toggle_ = toggle;
		if (buttonIcon != null) {
			setImage(buttonIcon, borderDim);
		}
		addMouseListener(new MouseListener() {
			// attribute

			boolean inside_ = false;
			boolean pushing_ = false;

			// event

			@Override
			public void mousePressed(MouseEvent e) {
				e.consume();
				if (!isEnabled()) {
					return;
				}
				if (!toggle_) {
					if (!pushing_) {
						pushing_ = true;
						if (inside_) {
							setPushed(true);
						}
					}
				} else {
					setPushed(!isPushed());
					fireClicked();
				}
			}

			@Override
			public void mouseReleased(MouseEvent e) {
				e.consume();
				if (!isEnabled()) {
					return;
				}
				if (pushing_) {
					pushing_ = false;
					if (inside_) {
						setPushed(false);
						fireClicked();
					}
				}
			}

			@Override
			public void mouseEntered(MouseEvent e) {
				e.consume();
				if (!isEnabled()) {
					return;
				}
				inside_ = true;
				if (pushing_) {
					setPushed(true);
				}
			}

			@Override
			public void mouseExited(MouseEvent e) {
				e.consume();
				if (!isEnabled()) {
					return;
				}
				inside_ = false;
				if (pushing_) {
					setPushed(false);
				}
			}

			@Override
			public void mouseClicked(MouseEvent e) {
				e.consume();
			}
		});
	}

	public GSButton(
		boolean toggle,
		BufferedImage buttonIcon)
	{
		this(toggle, buttonIcon, null);
	}

	public GSButton(
		boolean toggle)
	{
		this(toggle, null, null);
	}

	// property

	public boolean isPushed() {
		return pushed_;
	}

	public void setPushed(boolean pushed) {
		if (pushed != pushed_) {
			pushed_ = pushed;
			repaint();
		}
	}

	// operation

	public void setImage(
		BufferedImage buttonIcon,
		Dimension borderDim)
	{
		buttonIcon_ = buttonIcon;
		if (buttonIcon_ != null) {
			Dimension buttonDim = new Dimension(
				buttonIcon_.getWidth()+CX_BUTTONBORDER*2,
				buttonIcon_.getHeight()+CY_BUTTONBORDER*2);
			if (borderDim != null) {
				buttonDim.width +=
					(borderDim.width-CX_BUTTONBORDER)*2;
				buttonDim.height +=
					(borderDim.height-CY_BUTTONBORDER)*2;
			}
			setAllSize(buttonDim);
		}
	}
	public void setImage(
		BufferedImage buttonIcon)
	{
		setImage(buttonIcon, null);
	}

	// draw

	@Override
	public void paintComponent(Graphics g) {
		Color orgColor = g.getColor();
		Dimension buttonDim = getSize(null);
		Rectangle faceRect = new Rectangle(
			2, 2, buttonDim.width-4, buttonDim.height-4);
		if (!pushed_) {
			g.setColor(BORDER_HIGHLIGHT_COLOR1);
			g.fillRect(0, 0, buttonDim.width-1, 1);
			g.fillRect(0, 1, 1, buttonDim.height-2);
			g.setColor(BORDER_HIGHLIGHT_COLOR2);
			g.fillRect(1, 1, buttonDim.width-3, 1);
			g.fillRect(1, 2, 1, buttonDim.height-4);
			g.setColor(BORDER_SHADOW_COLOR1);
			g.fillRect(1, buttonDim.height-2, buttonDim.width-2, 1);
			g.fillRect(buttonDim.width-2, 1, 1, buttonDim.height-3);
			g.setColor(BORDER_SHADOW_COLOR2);
			g.fillRect(0, buttonDim.height-1, buttonDim.width, 1);
			g.fillRect(buttonDim.width-1, 0, 1, buttonDim.height);
		} else {
			g.setColor(BORDER_SHADOW_COLOR2);
			g.fillRect(0, 0, buttonDim.width, 1);
			g.fillRect(0, 1, 1, buttonDim.height-1);
			g.setColor(BORDER_HIGHLIGHT_COLOR1);
			g.fillRect(1, 1, buttonDim.width-1, 1);
			g.fillRect(1, 2, 1, buttonDim.height-1);
			g.setColor(BORDER_HIGHLIGHT_COLOR2);
			g.fillRect(2, 2, buttonDim.width-3, 1);
			g.fillRect(2, 3, 1, buttonDim.height-4);
			g.setColor(BORDER_SHADOW_COLOR1);
			g.fillRect(2, buttonDim.height-1, buttonDim.width-2, 1);
			g.fillRect(buttonDim.width-1, 2, 1, buttonDim.height-3);
			faceRect.x += 1;
			faceRect.y += 1;
		}
		if (g instanceof Graphics2D) {
			Graphics2D g2 = (Graphics2D)g;
			Paint orgPaint = g2.getPaint();
			g2.setPaint(
				new GradientPaint(
					faceRect.x, faceRect.y,
					FACE_COLOR1,
					faceRect.x, faceRect.y+faceRect.height,
					FACE_COLOR2));
			g2.fill(faceRect);
			g2.setPaint(orgPaint);
		} else {
			g.setColor(FACE_COLOR1);
			g.fillRect(
				faceRect.x, faceRect.y,
				faceRect.width, faceRect.height);
		}
		if (buttonIcon_ != null) {
			Point imagePt = new Point(
				(buttonDim.width-buttonIcon_.getWidth())/2,
				(buttonDim.height-buttonIcon_.getHeight())/2);
			if (pushed_) {
				imagePt.x += CX_ICONPUSHOFS;
				imagePt.y += CY_ICONPUSHOFS;
			}
			g.drawImage(
				buttonIcon_,
				imagePt.x, imagePt.y,
				null);
		}
		g.setColor(orgColor);
	}

	// button event

	public void addButtonListener(ButtonListener listener) {
		buttonListeners_.add(ButtonListener.class, listener);
	}

	public void removeButtonListener(ButtonListener listener) {
		buttonListeners_.remove(ButtonListener.class, listener);
	}

	void fireClicked() {
		ButtonEvent e = new ButtonEvent(this, getName(), pushed_);
		for (
			ButtonListener listener :
				buttonListeners_.getListeners(ButtonListener.class))
		{
			listener.buttonClicked(e);
		}
	}
}
