/**
 * GSweeper cell buttons panel
 *
 * @author Manuke
 */

package gsweeper;

// import

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;

import javax.swing.border.*;
import javax.swing.event.*;

// class implementation

class GSCellPanel
	extends GSPanel
{
	// type

	public class CellEvent
		extends EventObject
	{
		// attribute

		public int col;
		public int row;
		public boolean stat;

		// create & destroy

		public CellEvent(
			Object source,
			int col, int row, boolean stat)
		{
			super(source);
			this.col = col;
			this.row = row;
			this.stat = stat;
		}
	}

	public interface CellListener
		extends EventListener
	{
		// cell listener

		public void cellOpened(CellEvent e);
		public void cellMarked(CellEvent e);
		public void cellExOpened(CellEvent e);
		public void cellExOpening(CellEvent e);
	}

	// constant

	public static final int CX_ICONPUSHOFS = 1;
	public static final int CY_ICONPUSHOFS = 1;

	public static final Color CLOSED_BORDER_HIGHLIGHT_COLOR =
		new Color(0xC0C0C0C0, true);
	public static final Color CLOSED_BORDER_SHADOW_COLOR =
		new Color(0xC0808080, true);
	public static final Color CLOSED_FACE_COLOR =
		new Color(0xA0A0A0A0, true);
	public static final Color OPENED_BORDER_COLOR =
		new Color(0xC0202020, true);
	public static final Color OPENED_UNPUSHED_FACE_COLOR =
		new Color(0x80000000, true);
	public static final Color OPENED_PUSHED_FACE_COLOR =
		new Color(0xA0404040, true);
	public static final Color COMPLETED_FACE_COLOR =
		new Color(0x00000000, true);

	public static final Border CLOSED_UNPUSHED_BORDER =
		new BevelBorder(
			BevelBorder.RAISED,
			CLOSED_BORDER_HIGHLIGHT_COLOR,
			CLOSED_BORDER_SHADOW_COLOR);
	public static final Border CLOSED_PUSHED_BORDER =
		new BevelBorder(
			BevelBorder.LOWERED,
			CLOSED_BORDER_HIGHLIGHT_COLOR,
			CLOSED_BORDER_SHADOW_COLOR);
	public static final Border OPENED_BORDER =
		new LineBorder(OPENED_BORDER_COLOR);

	public static final float ICON_TRANSPARENT_RATE = 0.125f;

	// attribute

	GSGame game_;
	GSPictureManager pictman_;
	GSIconManager iconman_;
	int pushedCol_ = -1;
	int pushedRow_ = -1;

	boolean mousePushing_ = false;
	boolean mouseExOpening_ = false;
	boolean mouseMarking_ = false;
	int hoverCol_ = -1;
	int hoverRow_ = -1;
	int grabCol_ = -1;
	int grabRow_ = -1;

	Object sync_ = new Object();
	boolean dirty_ = true;
	boolean allDirty_ = true;
	boolean[][] cellDirties_;
	BufferedImage backBuffer_;

	EventListenerList cellListeners_ =
		new EventListenerList();

	// create & destroy

	public GSCellPanel(
		GSGame game,
		GSPictureManager pictman,
		GSIconManager iconman)
	{
		setOpaque(true);
		game_ = game;
		pictman_ = pictman;
		iconman_ = iconman;
		cellDirties_ =
			new boolean[game_.getRows()][game_.getCols()];
		for (int row = 0; row < game_.getRows(); row++) {
			for (int col = 0; col < game_.getCols(); col++) {
				cellDirties_[row][col] = true;
			}
		}
		Dimension pictureDim = game.getPictureDim();
		setAllSize(pictureDim);
		backBuffer_ = new BufferedImage(
			pictureDim.width, pictureDim.height,
			BufferedImage.TYPE_INT_ARGB);
		addMouseListener(new MouseListener() {
			// mouse listener

			@Override
			public void mousePressed(MouseEvent e) {
				e.consume();
				if (!isEnabled()) {
					return;
				}
				updateHoverCell(e.getX(), e.getY());
				if (mouseExOpening_) {
					return;
				}
				boolean push = false,
					exOpen = false,
					mark = false;
				int lrMask =
						MouseEvent.BUTTON1_DOWN_MASK |
						MouseEvent.BUTTON3_DOWN_MASK;
				if ((e.getButton() == MouseEvent.BUTTON2) ||
					((e.getModifiersEx() & lrMask) ==  lrMask) ||
					((e.getModifiersEx() & MouseEvent.META_DOWN_MASK) !=  0))
				{
					if (!mousePushing_) {
						push = true;
					}
					exOpen = true;
				} else if (!mousePushing_ && !mouseMarking_) {
					if ((e.getButton() == MouseEvent.BUTTON3) ||
						((e.getModifiersEx() & MouseEvent.CTRL_DOWN_MASK) !=  0))
					{
						mark = true;
					} else if (e.getButton() == MouseEvent.BUTTON1) {
						push = true;
					}
				}
				if (push) {
					mousePushing_ = true;
					if (grabCol_ < 0) {
						grabCol_ = hoverCol_;
						grabRow_ = hoverRow_;
					}
					if ((hoverCol_ == grabCol_) &&
						(hoverRow_ == grabRow_))
					{
						setCellPushed(grabCol_, grabRow_);
					}
				}
				if (exOpen) {
					mouseExOpening_ = true;
					if ((hoverCol_ == grabCol_) &&
						(hoverRow_ == grabRow_))
					{
						fireExOpening(grabCol_, grabRow_, true);
					}
				}
				if (mark) {
					mouseMarking_ = true;
					if (grabCol_ < 0) {
						grabCol_ = hoverCol_;
						grabRow_ = hoverRow_;
					}
					if ((hoverCol_ == grabCol_) &&
						(hoverRow_ == grabRow_))
					{
						fireMarked(grabCol_, grabRow_);
					}
				}
			}

			@Override
			public void mouseReleased(MouseEvent e) {
				e.consume();
				if (!isEnabled()) {
					return;
				}
				boolean mark = mouseMarking_;
				mouseMarking_ = false;
				if (mousePushing_) {
					mousePushing_ = false;
					boolean exOpen = mouseExOpening_;
					mouseExOpening_ = false;
					if ((hoverCol_ == grabCol_) &&
						(hoverRow_ == grabRow_))
					{
						setCellPushed(-1, -1);
						if (exOpen) {
							fireExOpening(grabCol_, grabRow_, false);
							fireExOpened(grabCol_, grabRow_);
						} else if (!mark) {
							fireOpened(grabCol_, grabRow_);
						}
					}
					grabCol_ = grabRow_ = -1;
				} else if (mark) {
					grabCol_ = grabRow_ = -1;
				}
			}

			@Override
			public void mouseEntered(MouseEvent e) {
				e.consume();
				if (!isEnabled()) {
					return;
				}
				updateHoverCell(e.getX(), e.getY());
				if (game_.isInside(hoverCol_, hoverRow_) &&
					(hoverCol_ == grabCol_) &&
					(hoverRow_ == grabRow_))
				{
					if (mousePushing_) {
						setCellPushed(grabCol_, grabRow_);
						if (mouseExOpening_) {
							fireExOpening(grabCol_, grabRow_, true);
						}
					}
				}
			}

			@Override
			public void mouseExited(MouseEvent e) {
				e.consume();
				if (!isEnabled()) {
					return;
				}
				updateHoverCell(-1, -1);
				if (mousePushing_) {
					setCellPushed(-1, -1);
					if (mouseExOpening_) {
						fireExOpening(grabCol_, grabRow_, false);
					}
				}
			}

			@Override
			public void mouseClicked(MouseEvent e) {
				e.consume();
			}
		});
		addMouseMotionListener(new MouseMotionListener() {
			// mouse motion listener

			@Override
			public void mouseMoved(MouseEvent e) {
				e.consume();
				if (!isEnabled()) {
					return;
				}
				updateHoverCell(e.getX(), e.getY());
				if (game_.isInside(hoverCol_, hoverRow_) &&
					(hoverCol_ == grabCol_) &&
					(hoverRow_ == grabRow_))
				{
					if (mousePushing_) {
						setCellPushed(grabCol_, grabRow_);
						if (mouseExOpening_) {
							fireExOpening(grabCol_, grabRow_, true);
						}
					}
				} else {
					if (mousePushing_) {
						setCellPushed(-1, -1);
						if (mouseExOpening_) {
							fireExOpening(grabCol_, grabRow_, false);
						}
					}
				}
			}

			@Override
			public void mouseDragged(MouseEvent e) {
				mouseMoved(e);
			}
		});
		game_.addMapListener(new GSGame.MapListener() {
			// map listener

			@Override
			public int getCol() {
				return -1;
			}

			@Override
			public int getRow() {
				return -1;
			}

			@Override
			public void mapChanged(GSGame.MapEvent e) {
				setCellDirty(e.col, e.row);
				repaintCell(e.col, e.row);
			}
		});
	}

	// operation

	protected void updateHoverCell(int x, int y) {
		Dimension pictureDim = game_.getPictureDim();
		if ((x >= 0) && (x < pictureDim.width) &&
			(y >= 0) && (y < pictureDim.height))
		{
			hoverCol_ = (x*game_.getCols())/pictureDim.width;
			hoverRow_ = (y*game_.getRows())/pictureDim.height;
			if (!game_.isInside(hoverCol_, hoverRow_)) {
				hoverCol_ = hoverRow_ = -1;
			}
		} else {
			hoverCol_ = hoverRow_ = -1;
		}
	}

	public void setCellPushed(int col, int row) {
		if ((col != pushedCol_) || (row != pushedRow_)) {
			int col2 = pushedCol_,
				row2 = pushedRow_;
			pushedCol_ = col;
			pushedRow_ = row;
			if (col2 >= 0) {
				setCellDirty(col2, row2);
				repaintCell(col2, row2);
			}
			if (col >= 0) {
				setCellDirty(col, row);
				repaintCell(col, row);
			}
		}
	}
	public void repaintCell(int col, int row) {
		repaint(game_.getCellRect(col, row));
	}

	// draw

	@Override
	public void paintComponent(Graphics g) {
		buildBackBuffer();
		g.drawImage(backBuffer_, 0, 0, null);
	}

	public void setAllDirty() {
		synchronized (sync_) {
			dirty_ = allDirty_ = true;
		}
	}

	public void setCellDirty(int col, int row) {
		if (game_.isInside(col, row)) {
			synchronized (sync_) {
				dirty_ = true;
				cellDirties_[row][col] = true;
			}
		}
	}

	protected void buildBackBuffer() {
		boolean dirty, allDirty;
		synchronized (sync_) {
			dirty = dirty_;
			allDirty = allDirty_;
			dirty_ = allDirty_ = false;
		}
		if (!dirty) {
			return;
		}
		Graphics g = backBuffer_.getGraphics();
		for (int row = 0; row < game_.getRows(); row++) {
			for (int col = 0; col < game_.getCols(); col++) {
				boolean cellDirty;
				synchronized (sync_) {
					cellDirty = cellDirties_[row][col];
					cellDirties_[row][col] = false;
				}
				if (!allDirty && !cellDirty) {
					continue;
				}
				drawCell(g, col, row);
			}
		}
		g.dispose();
	}

	protected void drawCell(Graphics g, int col, int row) {
		Color orgColor = g.getColor();
		boolean pushed = ((col == pushedCol_) && (row == pushedRow_));
		Rectangle cellRect = game_.getCellRect(col, row);
		BufferedImage compositePicture =
			pictman_.buildCompositePicture(game_);
		if (compositePicture != null) {
			g.drawImage(
				compositePicture,
				cellRect.x, cellRect.y,
				cellRect.x+cellRect.width, cellRect.y+cellRect.height,
				cellRect.x, cellRect.y,
				cellRect.x+cellRect.width, cellRect.y+cellRect.height,
				null);
		}
		int iconNo = -1;
		boolean transparent = false;
		if (game_.isPrepared(col, row)) {
			// no icon
		} else if (game_.isOpened(col, row)) {
			if (game_.isGhost(col, row)) {
				if (game_.isTalisman(col, row)) {
					iconNo = GSIconManager.ICON_EXGHOST;
				} else {
					iconNo = GSIconManager.ICON_GHOST;
				}
			} else if (game_.isTalisman(col, row)) {
				iconNo = GSIconManager.ICON_BADTALISMAN;
			} else {
				int aroundGhosts = game_.getAroundGhosts(col, row);
				if ((aroundGhosts >= 1) && (aroundGhosts <= 8)) {
					iconNo = GSIconManager.ICON_NUM1+aroundGhosts-1;
					transparent =
						(game_.getGameMode() == GSGame.GAMEMODE_COMPLETED);
				}
			}
		} else {
			if (game_.isTalisman(col, row)) {
				iconNo = GSIconManager.ICON_TALISMAN;
			} else if (game_.isQuestion(col, row)) {
				iconNo = GSIconManager.ICON_QUESTION;
			}
		}
		Border buttonBorder;
		Color faceColor;
		if (!game_.isOpened(col, row) &&
			!game_.isPrepared(col, row))
		{
			buttonBorder =
				pushed?
					CLOSED_PUSHED_BORDER:
				// else
					CLOSED_UNPUSHED_BORDER;
			faceColor = CLOSED_FACE_COLOR;
		} else {
			buttonBorder =  OPENED_BORDER;
			faceColor =
				(game_.getGameMode() == GSGame.GAMEMODE_COMPLETED)?
					COMPLETED_FACE_COLOR:
				pushed?
					OPENED_PUSHED_FACE_COLOR:
				// else
					OPENED_UNPUSHED_FACE_COLOR;
		}
		buttonBorder.paintBorder(
			this, g,
			cellRect.x, cellRect.y,
			cellRect.width, cellRect.height);
		Insets insets = buttonBorder.getBorderInsets(this);
		g.setColor(faceColor);
		g.fillRect(
			cellRect.x+insets.left,
			cellRect.y+insets.top,
			cellRect.width-(insets.left+insets.right),
			cellRect.height-(insets.top+insets.bottom));
		BufferedImage icon = null;
		if (iconNo >= 0) {
			icon = iconman_.getIcon(iconNo);
		}
		if (icon != null) {
			Point ptIcon = new Point(
				cellRect.x+(cellRect.width-icon.getWidth())/2,
				cellRect.y+(cellRect.height-icon.getHeight())/2);
			if (pushed) {
				ptIcon.x += CX_ICONPUSHOFS;
				ptIcon.y += CY_ICONPUSHOFS;
			}
			if (transparent &&
				(g instanceof Graphics2D))
			{
				Graphics2D g2 = (Graphics2D)g;
				Composite orgComposite = g2.getComposite();
				g2.setComposite(
					AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER,
						ICON_TRANSPARENT_RATE));
				g.drawImage(icon, ptIcon.x, ptIcon.y, null);
				g2.setComposite(orgComposite);
			} else {
				g.drawImage(icon, ptIcon.x, ptIcon.y, null);
			}
		}
		g.setColor(orgColor);
	}

	// cell event

	public void addCellListener(CellListener listener) {
		cellListeners_.add(CellListener.class, listener);
	}

	public void removeCellListener(CellListener listener) {
		cellListeners_.remove(CellListener.class, listener);
	}

	void fireOpened(int col, int row) {
		CellEvent e = new CellEvent(this, col, row, false);
		for (
			CellListener listener :
				cellListeners_.getListeners(CellListener.class))
		{
			listener.cellOpened(e);
		}
	}

	void fireMarked(int col, int row) {
		CellEvent e = new CellEvent(this, col, row, false);
		for (
			CellListener listener :
				cellListeners_.getListeners(CellListener.class))
		{
			listener.cellMarked(e);
		}
	}

	void fireExOpened(int col, int row) {
		CellEvent e = new CellEvent(this, col, row, false);
		for (
			CellListener listener :
				cellListeners_.getListeners(CellListener.class))
		{
			listener.cellExOpened(e);
		}
	}

	void fireExOpening(int col, int row, boolean stat) {
		CellEvent e = new CellEvent(this, col, row, stat);
		for (
			CellListener listener :
				cellListeners_.getListeners(CellListener.class))
		{
			listener.cellExOpening(e);
		}
	}
}
