/**
 * GSweeper generic panel
 *
 * @author Manuke
 */

package gsweeper;

// import

import java.awt.event.*;
import java.util.*;

import javax.swing.event.*;

// class implementation

class GSClickablePanel
	extends GSPanel
{
	// type

	public class PanelEvent
		extends EventObject
	{
		// attribute

		public String name;

		// create & destroy

		public PanelEvent(
			Object source,
			String name)
		{
			super(source);
			this.name = name;
		}
	}

	public interface PanelListener
		extends EventListener
	{
		// pannel listener

		public void panelClicked(PanelEvent e);
	}

	// attribute

	MouseListener curMouseListener_;

	EventListenerList panelListeners_ =
		new EventListenerList();

	// create & destroy

	public GSClickablePanel(boolean clickable) {
		curMouseListener_ = null;
		setClickable(clickable);
	}

	// clickable

	public boolean isClickable() {
		return curMouseListener_ != null;
	}
	public void setClickable(boolean clickable) {
		if (clickable == isClickable()) {
			// nothing
		} else if (clickable) {
			curMouseListener_ = new MouseListener() {
				// event

				@Override
				public void mousePressed(MouseEvent e) {
					e.consume();
					if (!isEnabled()) {
						return;
					}
					fireClicked();
				}

				@Override
				public void mouseReleased(MouseEvent e) {
					e.consume();
				}

				@Override
				public void mouseEntered(MouseEvent e) {
					e.consume();
				}

				@Override
				public void mouseExited(MouseEvent e) {
					e.consume();
				}

				@Override
				public void mouseClicked(MouseEvent e) {
					e.consume();
				}
			};
			addMouseListener(curMouseListener_);
		} else {
			removeMouseListener(curMouseListener_);
			curMouseListener_ = null;
		}
	}

	// panel event

	public void addPanelListener(PanelListener listener) {
		panelListeners_.add(PanelListener.class, listener);
	}

	public void removePanelListener(PanelListener listener) {
		panelListeners_.remove(PanelListener.class, listener);
	}

	void fireClicked() {
		PanelEvent e = new PanelEvent(this, getName());
		for (
			PanelListener listener :
				panelListeners_.getListeners(PanelListener.class))
		{
			listener.panelClicked(e);
		}
	}
}
