/**
 * GSweeper image operation utility
 *
 * @author Manuke
 */

package gsweeper;

// import

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;

import javax.imageio.*;

// class implementation

class GSImageUtility {
	// utility

	public static BufferedImage loadResourceImage(
		String name)
	{
		BufferedImage image;
		try {
			URL url = GSImageUtility.class.
				getClassLoader().getResource(name);
			image = ImageIO.read(url);
		} catch (IOException e) {
			image = null;
		}
		return image;
	}

	public static BufferedImage getFittedImage(
		BufferedImage orgImage,
		Dimension fittingDim,
		boolean expandable, boolean shrinkable)
	{
		if ((orgImage == null) ||
			(fittingDim == null) ||
			(fittingDim.width <= 0) ||
			(fittingDim.height <= 0) ||
			((orgImage.getWidth() == fittingDim.width) &&
				(orgImage.getHeight() == fittingDim.height)) ||
			(!expandable &&
				(orgImage.getWidth() <= fittingDim.width) &&
				(orgImage.getHeight() <= fittingDim.height)) ||
			(!shrinkable &&
				(orgImage.getWidth() >= fittingDim.width) &&
				(orgImage.getHeight() >= fittingDim.height)))
		{
			return orgImage;
		}
		Dimension orgDim = new Dimension(
			orgImage.getWidth(),
			orgImage.getHeight());
		Dimension newDim = new Dimension(
			(fittingDim.height*orgDim.width)/orgDim.height,
			(fittingDim.width*orgDim.height)/orgDim.width);
		if (newDim.width > fittingDim.width) {
			newDim.width = fittingDim.width;
		} else {
			newDim.height = fittingDim.height;
		}
		if (newDim.width < 1) {
			newDim.width = 1;
		}
		if (newDim.height < 1) {
			newDim.height = 1;
		}
		BufferedImage fittedImage = new BufferedImage(
			newDim.width, newDim.height,
			orgImage.getType());
		Graphics g = fittedImage.getGraphics();
		g.drawImage(
			orgImage.getScaledInstance(
				newDim.width, newDim.height,
				Image.SCALE_AREA_AVERAGING),
			0, 0,
			null);
		g.dispose();
		return fittedImage;
	}

	public static BufferedImage getHalfImage(
		BufferedImage orgImage,
		Dimension maxDim)
	{
		Dimension fittingDim = maxDim;
		if ((orgImage != null) && (maxDim != null)) {
			fittingDim = new Dimension(
				orgImage.getWidth()/2,
				orgImage.getHeight()/2);
			if (fittingDim.width > maxDim.width) {
				fittingDim.width = maxDim.width;
			}
			if (fittingDim.height > maxDim.height) {
				fittingDim.height = maxDim.height;
			}
			if (fittingDim.width < 1) {
				fittingDim.width = 1;
			}
			if (fittingDim.height < 1) {
				fittingDim.height = 1;
			}
		}
		return getFittedImage(orgImage, fittingDim, false, true);
	}
}
