/**
 * GSweeper information components panel
 *
 * @author Manuke
 */

package gsweeper;

// import

import java.awt.*;

import javax.swing.*;

// class implementation

class GSInfoPanel
	extends GSClickablePanel
{
	// constant

	public static final int CY_GAP = 20;
	public static final int CX_SPACE = 20;
	public static final int CY_SPACE = 20;
	public static final int CX_PROGRESS = 200;
	public static final int CY_PROGRESS = 40;
	public static final int CX_NPBUTTONBORDER = 10;
	public static final int CY_NPBUTTONBORDER = 10;

	public static final String TEXT_CLEAR_NAME =
		"image/text_clear.png";
	public static final String TEXT_GAMEOVER_NAME =
		"image/text_gameover.png";
	public static final String TEXT_TIMELIMIT_NAME =
		"image/text_timelimit.png";
	public static final String TEXT_NEXTPAGE_NAME =
		"image/text_nextpage.png";

	// attribute

	GSProgressBar loadingProgress_;
	GSImageView clearView_;
	GSButton nextPageButton_;
	GSImageView gameOverView_;
	GSImageView timeLimitView_;
	Box clearBox_;

	// create & destroy

	public GSInfoPanel(GSPictureManager pictman) {
		super(true);
		setOptimizedDrawingEnabled(false);
		Dimension pictureDim = pictman.getPictureDim(),
			maxDim = new Dimension(
				pictureDim.width-CX_SPACE*2,
				pictureDim.height-CY_SPACE*2),
			maxDim2 = new Dimension(
				maxDim.width-CX_NPBUTTONBORDER,
				maxDim.height-CY_NPBUTTONBORDER),
			progressDim = new Dimension(
				CX_PROGRESS,
				CY_PROGRESS);
		if (maxDim.width < 1) {
			maxDim.width = 1;
		}
		if (maxDim.height < 1) {
			maxDim.height = 1;
		}
		if (maxDim2.width < 1) {
			maxDim2.width = 1;
		}
		if (maxDim2.height < 1) {
			maxDim2.height = 1;
		}
		if (progressDim.width < 1) {
			progressDim.width = 1;
		}
		if (progressDim.height < 1) {
			progressDim.height = 1;
		}
		setAllSize(pictureDim);
		setLayout(new OverlayLayout(this));
		loadingProgress_ = new GSProgressBar();
		loadingProgress_.setAllSize(progressDim);
		add(loadingProgress_);
		loadingProgress_.setVisible(true);
		clearBox_ = Box.createVerticalBox();
		clearView_ = new GSImageView();
		clearView_.setImage(
			GSImageUtility.getHalfImage(
				GSImageUtility.loadResourceImage(TEXT_CLEAR_NAME),
				maxDim));
		clearView_.shrink();
		clearBox_.add(clearView_);
		clearView_.setVisible(true);
		clearBox_.add(Box.createVerticalStrut(CY_GAP));
		nextPageButton_ = new GSButton(
			false,
			GSImageUtility.getHalfImage(
				GSImageUtility.loadResourceImage(TEXT_NEXTPAGE_NAME),
				maxDim2),
			new Dimension(
				CX_NPBUTTONBORDER,
				CY_NPBUTTONBORDER));
		clearBox_.add(nextPageButton_);
		nextPageButton_.setVisible(false);
		add(clearBox_);
		clearBox_.setVisible(false);
		gameOverView_ = new GSImageView();
		gameOverView_.setImage(
			GSImageUtility.getHalfImage(
				GSImageUtility.loadResourceImage(TEXT_GAMEOVER_NAME),
				maxDim));
		gameOverView_.shrink();
		add(gameOverView_);
		gameOverView_.setVisible(false);
		timeLimitView_ = new GSImageView();
		timeLimitView_.setImage(
			GSImageUtility.getHalfImage(
				GSImageUtility.loadResourceImage(TEXT_TIMELIMIT_NAME),
				maxDim));
		timeLimitView_.shrink();
		add(timeLimitView_);
		timeLimitView_.setVisible(false);
		pictman.addLoadingListener(new GSPictureManager.LoadingListener() {
			@Override
			public void loadingUpdated(GSPictureManager.LoadingEvent e) {
				loadingProgress_.setStep(e.step);
				if (e.completed) {
					loadingProgress_.setVisible(false);
				}
			}
		});
	}

	// property

	GSProgressBar getLoadingProgress() {
		return loadingProgress_;
	}

	GSButton getNextPageButton() {
		return nextPageButton_;
	}

	// operation

	public void updateVisible(int gameMode) {
		switch (gameMode) {
		case GSGame.GAMEMODE_COMPLETED:
			clearBox_.setVisible(true);
			gameOverView_.setVisible(false);
			timeLimitView_.setVisible(false);
			setVisible(true);
			break;
		case GSGame.GAMEMODE_CURSED:
			clearBox_.setVisible(false);
			gameOverView_.setVisible(true);
			timeLimitView_.setVisible(false);
			setVisible(true);
			break;
		case GSGame.GAMEMODE_TIMELIMIT:
			clearBox_.setVisible(false);
			gameOverView_.setVisible(false);
			timeLimitView_.setVisible(true);
			setVisible(true);
			break;
		default:
			clearBox_.setVisible(false);
			gameOverView_.setVisible(false);
			timeLimitView_.setVisible(false);
			setVisible(false);
			break;
		}
	}
}
