/**
 * GSweeper main panel
 *
 * @author Manuke
 */

package gsweeper;

// import

import java.awt.*;

import javax.swing.*;
import javax.swing.border.*;

// class implementation

class GSMainPanel
	extends GSPanel
{
	// constant

	public static final int CX_GAP = 10;
	public static final int CY_GAP = 10;

	public static final Color BACKGROUND_COLOR1 =
		new Color(0xA0A0A0);
	public static final Color BACKGROUND_COLOR2 =
		new Color(0x606060);

	// attribute

	GSGame game_;
	GSController controller_;
	GSPictureManager pictman_;
	GSIconManager iconman_;
	GSDigitManager digitman_;

	// create & destroy

	public GSMainPanel(GSParameters params) {
		setOpaque(true);
		pictman_ = new GSPictureManager(
			params.picture1URI, params.picture2URI,
			params.mixMode);
		Dimension pictureDim = pictman_.getPictureDim();
		game_ = new GSGame(
			params.level,
			params.cols, params.rows, params.ghosts,
			params.times,
			pictureDim);
		iconman_ = new GSIconManager(
			game_.getCols(), game_.getRows(),
			pictureDim);
		digitman_ = new GSDigitManager();
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(Box.createVerticalGlue());
		GSToolPanel tools = new GSToolPanel(game_, digitman_);
		add(tools);
		tools.setVisible(true);
		add(Box.createVerticalGlue());
		GSPanel overlays = new GSPanel();
		overlays.setOptimizedDrawingEnabled(false);
		overlays.setLayout(
			new OverlayLayout(overlays));
		overlays.setBorder(
			new LineBorder(Color.BLACK));
		GSLogoPanel logos = new GSLogoPanel(game_);
		overlays.add(logos);
		logos.setVisible(true);
		GSInfoPanel infos = new GSInfoPanel(pictman_);
		overlays.add(infos);
		infos.setVisible(true);
		GSCellPanel cells = new GSCellPanel(game_, pictman_, iconman_);
		overlays.add(cells);
		cells.setVisible(true);
		GSPictureView pictureView = new GSPictureView(
			game_, pictman_);
		overlays.add(pictureView);
		pictureView.setVisible(false);
		add(overlays);
		overlays.setVisible(true);
		add(Box.createVerticalGlue());
		Dimension
			toolsDim = tools.getToolsDim(),
			mainDim = new Dimension(
				pictureDim.width+CX_GAP*2,
				pictureDim.height+GSToolPanel.CY_TOOLS+CY_GAP*3);
		toolsDim.width += CX_GAP*2;
		if (toolsDim.width > mainDim.width) {
			mainDim.width = toolsDim.width;
		}
		setMinimumSize(mainDim);
		setPreferredSize(mainDim);
		controller_ = new GSController(
			game_,
			pictman_,
			tools, logos, infos, cells,
			pictureView,
			params.uriProcessor, params.nextURI,
			params.randomSeed);
	}

	// paint

	@Override
	public void paintComponent(Graphics g) {
		Dimension mainDim = getSize();
		if (g instanceof Graphics2D) {
			Graphics2D g2 = (Graphics2D)g;
			Paint orgPaint = g2.getPaint();
			g2.setPaint(
				new GradientPaint(
					0, 0, BACKGROUND_COLOR1,
					0, mainDim.height, BACKGROUND_COLOR2));
			g2.fill(new Rectangle(0, 0, mainDim.width, mainDim.height));
			g2.setPaint(orgPaint);
		} else {
			Color orgColor = g.getColor();
			g.setColor(BACKGROUND_COLOR1);
			g.fillRect(0, 0, mainDim.width, mainDim.height);
			g.setColor(orgColor);
		}
	}
}
