/**
 * GSweeper number view component
 *
 * @author Manuke
 */

package gsweeper;

// import

import java.awt.*;
import java.awt.image.*;

import javax.swing.border.*;

// class implementation

class GSNumberView
	extends GSComponent
{
	// constant

	public static final int CX_BORDER = 4;
	public static final int CY_BORDER = 4;

	public static final Color BORDER_HIGHLIGHT_COLOR =
		new Color(0xE0E0E0);
	public static final Color BORDER_SHADOW_COLOR =
		new Color(0xA0A0A0);
	public static final Color BACKGROUND_COLOR =
		new Color(0x000000);

	public static final Border NUMBER_BORDER =
		new BevelBorder(
			BevelBorder.LOWERED,
			BORDER_HIGHLIGHT_COLOR,
			BORDER_SHADOW_COLOR);

	// attribute

	GSDigitManager digitman_;
	int cols_;
	int number_;

	// create & destroy

	public GSNumberView(
		GSDigitManager digitman,
		int cols)
	{
		setOpaque(true);
		digitman_ = digitman;
		cols_ = cols;
		number_ = -1;
		setBorder(NUMBER_BORDER);
		Dimension numberDim = getNumberDim();
		setAllSize(numberDim);
	}

	// property

	public int getCols() {
		return cols_;
	}

	public int getNumber() {
		return number_;
	}

	public void setNumber(int number) {
		if (number != number_) {
			number_ = number;
			repaint();
		}
	}

	public Dimension getNumberDim() {
		Dimension digitDim = digitman_.getDigitDim();
		return new Dimension(
			digitDim.width*cols_+CX_BORDER*2,
			digitDim.height+CY_BORDER*2);
	}

	// draw

	@Override
	public void paintComponent(Graphics g) {
		Color orgColor = g.getColor();
		Dimension digitDim = digitman_.getDigitDim();
		Dimension numberDim = getSize(null);
		String numberStr = String.valueOf(number_);
		if (numberStr.length() > cols_) {
			numberStr = numberStr.substring(
				numberStr.length()-cols_);
		}
		Insets insets = getInsets();
		g.setColor(BACKGROUND_COLOR);
		g.fillRect(
			insets.left,
			insets.top,
			numberDim.width-(insets.left+insets.right),
			numberDim.height-(insets.top+insets.bottom));
		if (number_ >= 0) {
			int x = (numberDim.width-digitDim.width*cols_)/2+
						digitDim.width*(cols_-1),
				y = (numberDim.height-digitDim.height)/2;
			for (int col = 0; col < numberStr.length(); col++) {
				int digit = Character.digit(
						numberStr.charAt(numberStr.length()-col-1),
						10);
				BufferedImage digitImage = digitman_.getDigitImage(digit);
				if (digitImage != null) {
					g.drawImage(digitImage, x, y, null);
				}
				x -= digitDim.width;
			}
		}
		g.setColor(orgColor);
	}
}
