/**
 * GSweeper progress bar component
 *
 * @author Manuke
 */

package gsweeper;

// import

import java.awt.*;

import javax.swing.border.*;

// class implementation

class GSProgressBar
	extends GSComponent
{
	// constant

	public static final Color BORDER_HIGHLIGHT_COLOR =
		new Color(0xE0E0E0);
	public static final Color BORDER_SHADOW_COLOR =
		new Color(0xA0A0A0);
	public static final Color BACKGROUND_COLOR =
		new Color(0x808080);
	public static final Color PROGRESS_COLOR1 =
		new Color(0x4040FF);
	public static final Color PROGRESS_COLOR2 =
		new Color(0x80C0FF);

	public static final Border PROGRESS_BORDER =
		new BevelBorder(
			BevelBorder.LOWERED,
			BORDER_HIGHLIGHT_COLOR,
			BORDER_SHADOW_COLOR);

	// attribute

	float step_ = 0f;

	// create & destroy

	public GSProgressBar() {
		setOpaque(true);
		setBorder(PROGRESS_BORDER);
	}

	// property

	public float getStep() {
		return step_;
	}

	public void setStep(float step) {
		if (step != step_) {
			step_ = step;
			repaint();
		}
	}

	// draw

	@Override
	public void paintComponent(Graphics g) {
		Color orgColor = g.getColor();
		Dimension progressDim = getSize(null);
		Insets insets = getInsets();
		Rectangle drawRect = new Rectangle(
			insets.left,
			insets.top,
			progressDim.width-(insets.left+insets.right),
			progressDim.height-(insets.top+insets.bottom));
		g.setColor(BACKGROUND_COLOR);
		g.fillRect(
			drawRect.x,
			drawRect.y,
			drawRect.width,
			drawRect.height);
		int progressWidth = (int)(drawRect.width*step_+0.5);
		if (progressWidth > drawRect.width) {
			progressWidth = drawRect.width;
		}
		if (progressWidth > 0) {
			if (g instanceof Graphics2D) {
				Graphics2D g2 = (Graphics2D)g;
				Paint orgPaint = g2.getPaint();
				g2.setPaint(
					new GradientPaint(
						drawRect.x, drawRect.y,
						PROGRESS_COLOR1,
						drawRect.x+drawRect.width, drawRect.y,
						PROGRESS_COLOR2));
				g.fillRect(
					drawRect.x,
					drawRect.y,
					progressWidth,
					drawRect.height);
				g2.setPaint(orgPaint);
			} else {
				g.setColor(PROGRESS_COLOR1);
				g.fillRect(
					drawRect.x,
					drawRect.y,
					progressWidth,
					drawRect.height);
			}
		}
		g.setColor(orgColor);
	}
}
