/**
 * GSweeper tool components panel
 *
 * @author Manuke
 */

package gsweeper;

// import

import java.awt.*;

import javax.swing.*;

// class implementation

class GSToolPanel
	extends GSPanel
{
	// constant

	public static final int CX_BUTTON = 40;
	public static final int CY_BUTTON = 40;
	public static final int CY_TOOLS = 40;
	public static final int CX_GAP = 10;
	public static final int CX_BUTTONBORDER = 4;
	public static final int CY_BUTTONBORDER = 4;
	public static final int CX_MARK = CX_BUTTON-CX_BUTTONBORDER*2;
	public static final int CY_MARK = CY_BUTTON-CY_BUTTONBORDER*2;

	public static final String ICON_START_NAME =
		"image/icon_start.png";
	public static final String ICON_SCOPE_NAME =
		"image/icon_scope.png";
	public static final String ICON_TALISMAN_NAME =
		"image/icon_talisman.png";
	public static final String ICON_STOPWATCH_NAME =
		"image/icon_stopwatch.png";

	// sttribute

	GSButton startButton_;
	GSButton scopeButton_;
	GSNumberView talismansView_;
	GSNumberView timeView_;

	// create & destroy

	public GSToolPanel(
		GSGame game,
		GSDigitManager digitman)
	{
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		Dimension markDim = new Dimension(CX_MARK, CY_MARK);
		startButton_ = new GSButton(
			false,
			GSImageUtility.getFittedImage(
				GSImageUtility.loadResourceImage(ICON_START_NAME),
				markDim,
				true, true));
		add(startButton_);
		startButton_.setVisible(true);
		add(Box.createHorizontalStrut(CX_GAP));
		GSImageView talismanMark = new GSImageView(
			GSImageUtility.getFittedImage(
				GSImageUtility.loadResourceImage(ICON_TALISMAN_NAME),
				markDim,
				true, true));
		add(talismanMark);
		talismanMark.setVisible(true);
		talismansView_ = new GSNumberView(
			digitman,
			GSDigitManager.adjustCols(3, game.getGhosts()));
		add(talismansView_);
		talismansView_.setVisible(true);
		add(Box.createHorizontalStrut(CX_GAP));
		GSImageView stopwatchMark = new GSImageView(
			GSImageUtility.getFittedImage(
				GSImageUtility.loadResourceImage(ICON_STOPWATCH_NAME),
				markDim,
				true, true));
		add(stopwatchMark);
		stopwatchMark.setVisible(true);
		int times = game.getTimes();
		if (times < 0) {
			times = 0;
		}
		timeView_ = new GSNumberView(
			digitman,
			GSDigitManager.adjustCols(3, times));
		add(timeView_);
		timeView_.setVisible(true);
		add(Box.createHorizontalStrut(CX_GAP));
		scopeButton_ = new GSButton(
			true,
			GSImageUtility.getFittedImage(
				GSImageUtility.loadResourceImage(ICON_SCOPE_NAME),
				markDim,
				true, true));
		add(scopeButton_);
		scopeButton_.setVisible(true);
	}

	// property

	GSButton getStartButton() {
		return startButton_;
	}

	GSButton getScopeButton() {
		return scopeButton_;
	}

	GSNumberView getTalismansView() {
		return talismansView_;
	}

	GSNumberView getTimeView() {
		return timeView_;
	}

	Dimension getToolsDim() {
		Dimension talismansDim = talismansView_.getNumberDim(),
			timeDim = timeView_.getNumberDim(),
			toolsDim = new Dimension(
				talismansDim.width+
					timeDim.width+
					CX_BUTTON*2+
					CX_MARK*2+
					CX_GAP*3,
				CY_TOOLS);
		return toolsDim;
	}
}
